@@ -12,7 +12,6 @@
 #
 ##############################################################################
 import datetime
-import decimal
 import doctest
 import unittest
 
@@ -314,62 +313,6 @@ class FloatTests(NumberTests):
         self.assertEqual(flt.fromUnicode(u'1.23e6'), 1230000.0)
 
 
-class DecimalTests(NumberTests):
-
-    mvm_missing_value = decimal.Decimal("-1")
-    mvm_default = decimal.Decimal("0")
-
-    MIN = decimal.Decimal(NumberTests.MIN)
-    MAX = decimal.Decimal(NumberTests.MAX)
-    VALID = tuple(decimal.Decimal(x) for x in NumberTests.VALID)
-    TOO_SMALL = tuple(decimal.Decimal(x) for x in NumberTests.TOO_SMALL)
-    TOO_BIG = tuple(decimal.Decimal(x) for x in NumberTests.TOO_BIG)
-
-    def _getTargetClass(self):
-        from zope.schema._field import Decimal
-        return Decimal
-
-    def _getTargetInterface(self):
-        from zope.schema.interfaces import IDecimal
-        return IDecimal
-
-    def test_validate_not_required(self):
-        field = self._makeOne(required=False)
-        field.validate(decimal.Decimal("10.0"))
-        field.validate(decimal.Decimal("0.93"))
-        field.validate(decimal.Decimal("1000.0003"))
-        field.validate(None)
-
-    def test_validate_required(self):
-        from zope.schema.interfaces import RequiredMissing
-        field = self._makeOne()
-        field.validate(decimal.Decimal("10.0"))
-        field.validate(decimal.Decimal("0.93"))
-        field.validate(decimal.Decimal("1000.0003"))
-        self.assertRaises(RequiredMissing, field.validate, None)
-
-    def test_fromUnicode_miss(self):
-        from zope.schema.interfaces import ValidationError
-        flt = self._makeOne()
-        self.assertRaises(ValueError, flt.fromUnicode, u'')
-        self.assertRaises(ValueError, flt.fromUnicode, u'abc')
-        with self.assertRaises(ValueError) as exc:
-            flt.fromUnicode(u'1.4G')
-
-        value_error = exc.exception
-        self.assertIs(value_error.field, flt)
-        self.assertEqual(value_error.value, u'1.4G')
-        self.assertIsInstance(value_error, ValidationError)
-
-    def test_fromUnicode_hit(self):
-        from decimal import Decimal
-
-        flt = self._makeOne()
-        self.assertEqual(flt.fromUnicode(u'0'), Decimal('0.0'))
-        self.assertEqual(flt.fromUnicode(u'1.23'), Decimal('1.23'))
-        self.assertEqual(flt.fromUnicode(u'12345.6'), Decimal('12345.6'))
-
-
 class DatetimeTests(OrderableMissingValueMixin,
                     OrderableTestsMixin,
                     EqualityTestsMixin,
@@ -768,7 +711,7 @@ class ChoiceTests(EqualityTestsMixin,
         from zope.schema.vocabulary import setVocabularyRegistry
 
         class Reg(object):
-            def get(*args):
+            def get(self, *args):
                 raise LookupError
 
         setVocabularyRegistry(Reg())
@@ -914,6 +857,51 @@ class URITests(EqualityTestsMixin,
                           field.fromUnicode, u'http://example.com/\nDAV:')
 
 
+class PythonIdentifierTests(EqualityTestsMixin,
+                            WrongTypeTestsMixin,
+                            unittest.TestCase):
+
+    def _getTargetClass(self):
+        from zope.schema._field import PythonIdentifier
+        return PythonIdentifier
+
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
+    def _getTargetInterface(self):
+        from zope.schema.interfaces import IPythonIdentifier
+        return IPythonIdentifier
+
+    def test_fromUnicode_empty(self):
+        pi = self._makeOne()
+        self.assertEqual(pi.fromUnicode(u''), '')
+
+    def test_fromUnicode_normal(self):
+        pi = self._makeOne()
+        self.assertEqual(pi.fromUnicode(u'normal'), 'normal')
+
+    def test_fromUnicode_strips_ws(self):
+        pi = self._makeOne()
+        self.assertEqual(pi.fromUnicode(u'   '), '')
+        self.assertEqual(pi.fromUnicode(u' normal  '), 'normal')
+
+    def test__validate_miss(self):
+        from zope.schema.interfaces import InvalidValue
+        pi = self._makeOne()
+        with self.assertRaises(InvalidValue) as exc:
+            pi._validate('not-an-identifier')
+
+        ex = exc.exception
+        self.assertIs(ex.field, pi)
+        self.assertEqual(ex.value, 'not-an-identifier')
+
+    def test__validate_hit(self):
+        pi = self._makeOne()
+        pi._validate('is_an_identifier')
+
+
 class DottedNameTests(EqualityTestsMixin,
                       WrongTypeTestsMixin,
                       unittest.TestCase):
@@ -1016,6 +1004,12 @@ class DottedNameTests(EqualityTestsMixin,
         field = self._makeOne()
         self.assertEqual(field.fromUnicode(u'dotted.name'), 'dotted.name')
 
+        # Underscores are allowed in any component
+        self.assertEqual(field.fromUnicode(u'dotted._name'), 'dotted._name')
+        self.assertEqual(field.fromUnicode(u'_leading_underscore'), '_leading_underscore')
+        self.assertEqual(field.fromUnicode(u'_dotted.name'), '_dotted.name')
+        self.assertEqual(field.fromUnicode(u'_dotted._name'), '_dotted._name')
+
     def test_fromUnicode_invalid(self):
         from zope.schema.interfaces import ConstraintNotSatisfied
         from zope.schema.interfaces import InvalidDottedName
@@ -1375,7 +1369,7 @@ class SequenceTests(WrongTypeTestsMixin,
         from zope.schema._field import abc
 
         class MutableSequence(abc.MutableSequence):
-            def insert(self, item, ix):
+            def insert(self, index, value):
                 raise AssertionError("not implemented")
             def __getitem__(self, name):
                 raise AssertionError("not implemented")
@@ -1702,6 +1696,78 @@ class DictTests(MutableMappingTests):
         with self.assertRaises(WrongType):
             super(DictTests, self).test_mutable_mapping()
 
+class NativeStringTests(EqualityTestsMixin,
+                        WrongTypeTestsMixin,
+                        unittest.TestCase):
+
+    def _getTargetClass(self):
+        from zope.schema._field import NativeString
+        return NativeString
+
+    def _getTargetInterface(self):
+        from zope.schema.interfaces import INativeString
+        return INativeString
+
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
+    def test_fromBytes(self):
+        field = self._makeOne()
+        self.assertEqual(field.fromBytes(b''), '')
+        self.assertEqual(field.fromBytes(b'DEADBEEF'), 'DEADBEEF')
+
+    def test_fromUnicode(self):
+        field = self._makeOne()
+        self.assertIsInstance(field.fromUnicode(u''), str)
+        self.assertEqual(field.fromUnicode(u''), '')
+        self.assertEqual(field.fromUnicode(u'DEADBEEF'), 'DEADBEEF')
+
+
+class NativeStringLineTests(EqualityTestsMixin,
+                            WrongTypeTestsMixin,
+                            unittest.TestCase):
+
+    def _getTargetClass(self):
+        from zope.schema._field import NativeStringLine
+        return NativeStringLine
+
+    def _getTargetInterface(self):
+        from zope.schema.interfaces import INativeStringLine
+        return INativeStringLine
+
+    def _getTargetInterfaces(self):
+        from zope.schema.interfaces import IFromUnicode
+        from zope.schema.interfaces import IFromBytes
+        return [self._getTargetInterface(), IFromUnicode, IFromBytes]
+
+    def test_fromBytes(self):
+        field = self._makeOne()
+        self.assertEqual(field.fromBytes(b''), '')
+        self.assertEqual(field.fromBytes(b'DEADBEEF'), 'DEADBEEF')
+
+    def test_fromUnicode(self):
+        field = self._makeOne()
+        self.assertIsInstance(field.fromUnicode(u''), str)
+        self.assertEqual(field.fromUnicode(u''), '')
+        self.assertEqual(field.fromUnicode(u'DEADBEEF'), 'DEADBEEF')
+
+
+class StrippedNativeStringLineTests(NativeStringLineTests):
+
+    def _getTargetClass(self):
+        from zope.schema._field import _StrippedNativeStringLine
+        return _StrippedNativeStringLine
+
+    def test_strips(self):
+        field = self._makeOne()
+        self.assertEqual(field.fromBytes(b' '), '')
+        self.assertEqual(field.fromUnicode(u' '), '')
+
+    def test_iface_is_first_in_sro(self):
+        self.skipTest("Not applicable; we inherit implementation but have no interface")
+
 
 def _makeSampleVocabulary():
     from zope.interface import implementer
@@ -1733,7 +1799,7 @@ def _makeDummyRegistry(v):
             VocabularyRegistry.__init__(self)
             self._vocabulary = vocabulary
 
-        def get(self, object, name):
+        def get(self, context, name):
             return self._vocabulary
     return DummyRegistry(v)
 
